﻿namespace TabsStudioSorter
{
    public delegate void DragDropHandler(int sourceIndex, int targetIndex);

    class ListViewDragManager
    {
        public ListViewDragManager(System.Windows.Controls.ListView listView_)
        {
            listView = listView_;
            cxdrag = System.Windows.SystemParameters.MinimumHorizontalDragDistance > 0 ?
	            System.Windows.SystemParameters.MinimumHorizontalDragDistance : 4.0;
            cydrag = System.Windows.SystemParameters.MinimumVerticalDragDistance > 0 ?
                System.Windows.SystemParameters.MinimumVerticalDragDistance : 4.0;
            listView.AllowDrop = true;
            listView.MouseMove += OnMouseMove;
            listView.DragEnter += OnCheckDropTarget;
            listView.DragLeave += OnCheckDropTarget;
            listView.DragOver += OnCheckDropTarget;
            listView.Drop += OnDrop;

            // MouseDown doesn't fire when clicking a list item
            listView.PreviewMouseLeftButtonDown += OnPreviewMouseLeftButtonDown;

            clickedIndex = -1;
            targetIndex = -1;
        }

        public event DragDropHandler OnDragDrop;

        private void OnPreviewMouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            clickedIndex = -1;
            targetIndex = -1;
		    for (int i = 0; i < listView.Items.Count; ++i)
		    {
			    System.Windows.Controls.ListViewItem item = GetListViewItem(i);
			    if (item != null && item.IsMouseOver)
			    {
                    clickedIndex = i;
				    lastMouseDown = e.GetPosition(listView);
				    break;
			    }
		    }
        }

        private void OnMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
		    if (e.LeftButton == System.Windows.Input.MouseButtonState.Pressed && clickedIndex >= 0 && 
			    listView.SelectedIndex == clickedIndex)
		    {
			    System.Windows.Point currentPosition = e.GetPosition(listView);
			    if ((System.Math.Abs(currentPosition.X - lastMouseDown.X) > cxdrag) ||
				    (System.Math.Abs(currentPosition.Y - lastMouseDown.Y) > cydrag))
			    {
				    targetIndex = -1;
				    System.Windows.DragDropEffects finalDropEffect = System.Windows.DragDrop.DoDragDrop(
							    listView, listView.SelectedItem, System.Windows.DragDropEffects.Move);
				    if (finalDropEffect == System.Windows.DragDropEffects.Move && targetIndex >= 0 &&
					    clickedIndex != targetIndex)
				    {
                        if (OnDragDrop != null)
					        OnDragDrop(clickedIndex, targetIndex);
				    }
                    clickedIndex = -1;
                    targetIndex = -1;
                }
		    }
        }

        private void OnCheckDropTarget(object sender, System.Windows.DragEventArgs e)
        {
		    e.Effects =
                (clickedIndex >= 0 && FindListViewItem(e.OriginalSource as System.Windows.UIElement) != null) ?
			    System.Windows.DragDropEffects.Move :
			    System.Windows.DragDropEffects.None;
		    e.Handled = true;
        }


        private void OnDrop(object sender, System.Windows.DragEventArgs e)
        {
		    bool valid = false;
            System.Windows.Controls.ListViewItem item = FindListViewItem(e.OriginalSource as System.Windows.UIElement);
		    if (clickedIndex >= 0 && item != null)
		    {
			    for (int i = 0; i < listView.Items.Count; ++i)
			    {
				    if (GetListViewItem(i) == item)
				    {
					    targetIndex = i;
					    valid = true;
					    break;
				    }
			    }
		    }
		    e.Effects = valid ?
			    System.Windows.DragDropEffects.Move :
			    System.Windows.DragDropEffects.None;
		    e.Handled = true;
        }

        private System.Windows.Controls.ListViewItem GetListViewItem(int index)
        {
	        if(listView.ItemContainerGenerator.Status != System.Windows.Controls.Primitives.GeneratorStatus.ContainersGenerated)
		        return null;

	        return listView.ItemContainerGenerator.ContainerFromIndex(index) as System.Windows.Controls.ListViewItem;
        }

	    private System.Windows.Controls.ListViewItem FindListViewItem(System.Windows.UIElement element)
	    {
		    while (element != null)
		    {
			    System.Windows.Controls.ListViewItem item = element as System.Windows.Controls.ListViewItem;
			    if (item != null)
				    return item;
			    element = System.Windows.Media.VisualTreeHelper.GetParent(element) as System.Windows.UIElement;
		    } 
		    return null;
	    }

        private System.Windows.Controls.ListView listView;
        private int clickedIndex, targetIndex;
	    private System.Windows.Point lastMouseDown;
        private double cxdrag, cydrag;
    }
}
