﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace TabsStudioSorter
{
    /// <summary>
    /// Interaction logic for SettingsDialog.xaml
    /// </summary>
    public partial class OptionsDialog
    {
        public OptionsDialog(Options options_)
        {
            options = options_;

            rules = new System.Collections.Generic.List<SortingRule>();
            foreach (SortingRule rule in options.rules)
                rules.Add(rule.Clone());

            InitializeComponent();
            UpdateRules();

            rulesList.SelectionChanged += OnSelectionChanged;
            UpdateRuleButtonsStatus();

            dragManager = new ListViewDragManager(rulesList);
            dragManager.OnDragDrop += OnDragDrop;
        }

        private void OnDragDrop(int sourceIndex, int targetIndex)
        {
            SortingRule source = rules[sourceIndex];
            rules.RemoveAt(sourceIndex);
            rules.Insert(targetIndex, source);
            UpdateRules();
        }

        private void OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            UpdateRuleButtonsStatus();
        }

        private void UpdateRuleButtonsStatus()
        {
	        int i = rulesList.SelectedIndex;
	        if (i < 0)
	        {
		        buttonEdit.IsEnabled = false;
		        buttonClone.IsEnabled = false;
		        buttonDelete.IsEnabled = false;
		        return;
	        }
            buttonEdit.IsEnabled = true;
            buttonClone.IsEnabled = true;
            buttonDelete.IsEnabled = true;
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            options.rules = rules;
            DialogResult = true;
        }

        private void buttonAdd_Click(object sender, RoutedEventArgs e)
        {
            SortingRule rule = new SortingRule("", SortingRuleType.TabName, "(?<M>^.*$)", false);
            EditRuleDialog dialog = new EditRuleDialog(rule);
            dialog.Owner = this;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
            {
                rules.Add(rule);
                UpdateRules();
            }
        }

        private void buttonEdit_Click(object sender, RoutedEventArgs e)
        {
            EditRuleDialog dialog = new EditRuleDialog(rules[rulesList.SelectedIndex]);
            dialog.Owner = this;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
            {
                UpdateRules();
            }
        }

        private void buttonClone_Click(object sender, RoutedEventArgs e)
        {
    		SortingRule clone = rules[rulesList.SelectedIndex].Clone();
    		clone.name = "Clone of " + clone.name;
		    rules.Add(clone);
		    UpdateRules();
        }

        private void buttonDelete_Click(object sender, RoutedEventArgs e)
        {
            rules.RemoveAt(rulesList.SelectedIndex);
            UpdateRules();
        }

        private void UpdateRules()
        {
            rulesList.UpdateRules(rules);
        }
        
        private Options options;
        private System.Collections.Generic.List<SortingRule> rules;
        private ListViewDragManager dragManager;
    }
}
