﻿namespace TabsStudioSorter
{
    class PriorityGroup
    {
        public delegate void TopChangedDelegate(System.Windows.Controls.TabItem tabItem, System.Windows.Controls.Panel panel);
        public event TopChangedDelegate OnTopChanged;

        public PriorityGroup()
        {
            priorityGroupNotFound = false;
            openTabs = new System.Collections.Generic.List<TabInfo>();
        }

        public bool IsPriorityTab(System.Windows.Controls.TabItem tabItem)
        {
            System.ComponentModel.DependencyPropertyDescriptor d = GetTopDescriptor(tabItem);
            return d != null && (bool)d.GetValue(tabItem);
        }

        public void OnTabCreated(TabsStudioExt.TabEventArgs e)
        {
            TabInfo info = new TabInfo(e.Tab.TabItem,
                (object o, System.EventArgs a) =>
                    { if (OnTopChanged != null) OnTopChanged(e.Tab.TabItem, e.Tabs.TabPanel2); });
            openTabs.Add(info);
            System.ComponentModel.DependencyPropertyDescriptor d = GetTopDescriptor(e.Tab.TabItem);
            if (d != null)
                d.AddValueChanged(info.item, info.topChanged);
        }

        public void OnTabDestroyed(TabsStudioExt.TabEventArgs e)
        {
            foreach (TabInfo info in openTabs)
            {
                if (info.item == e.Tab.TabItem)
                {
                    System.ComponentModel.DependencyPropertyDescriptor d = GetTopDescriptor(e.Tab.TabItem);
                    if (d != null)
                        d.RemoveValueChanged(info.item, info.topChanged);
                    openTabs.Remove(info);
                    return;
                }
            }
        }

        private System.ComponentModel.DependencyPropertyDescriptor GetTopDescriptor(System.Windows.Controls.TabItem tabItem)
        {
            if (topDescriptorCached == null && !priorityGroupNotFound)
            {
                priorityGroupNotFound = true;
                foreach (System.Reflection.Assembly a in System.AppDomain.CurrentDomain.GetAssemblies())
                {
                    if (a.FullName.StartsWith("PriorityGroup,"))
                    {
                        System.Type t = a.GetType("TabsStudioPriorityGroup.Properties");
                        if (t != null)
                            topDescriptorCached = System.ComponentModel.DependencyPropertyDescriptor.FromName(
                                "Top", t, tabItem.GetType());
                        break;
                    }
                }
            }
            return topDescriptorCached;
        }

        private System.ComponentModel.DependencyPropertyDescriptor topDescriptorCached;
        private bool priorityGroupNotFound;
        private System.Collections.Generic.List<TabInfo> openTabs;
    }
}
